﻿using System;
using System.Security.Cryptography;
using System.Text;
using System.IO;
using Arawn.Extensions;
using Arawn.Modules.Cryptography.Pages;
using Xamarin.Forms;
using System.Threading.Tasks;
using Arawn.Modules.Storage;
using Newtonsoft.Json;
using System.Collections.ObjectModel;
using Arawn.Modules.Storage.Models;

namespace Arawn.Modules.Cryptography
{
    static public class CryptographyHelper
    {
        public static string PASSWORD;

        public static string Encrypt(string plainText, string password)
        {
            if (String.IsNullOrEmpty(plainText)) return "";
            if (String.IsNullOrEmpty(password)) return plainText;
            password = password.Reverse();

            var algorithm = GetAlgorithm(password);

            byte[] encryptedBytes;
            using (ICryptoTransform encryptor = algorithm.CreateEncryptor(algorithm.Key, algorithm.IV))
            {
                byte[] bytesToEncrypt = Encoding.UTF8.GetBytes(plainText);
                encryptedBytes = StreamTransform(bytesToEncrypt, encryptor);
            }
            return Convert.ToBase64String(encryptedBytes);
        }

        public static string Decrypt(string encryptedText, string password)
        {
            if (String.IsNullOrEmpty(encryptedText)) return "";
            if (String.IsNullOrEmpty(password)) return encryptedText;
            password = password.Reverse();

            var algorithm = GetAlgorithm(password);

            byte[] descryptedBytes;
            using (ICryptoTransform decryptor = algorithm.CreateDecryptor(algorithm.Key, algorithm.IV))
            {
                byte[] encryptedBytes = Convert.FromBase64String(encryptedText);
                descryptedBytes = StreamTransform(encryptedBytes, decryptor);
            }
            return Encoding.UTF8.GetString(descryptedBytes);
        }

        private static RijndaelManaged GetAlgorithm(string password)
        {
            var key = new Rfc2898DeriveBytes(password, Encoding.ASCII.GetBytes("4r4Wn P455w0Rd 541T"));

            var rima = new RijndaelManaged();
            rima.Key = key.GetBytes(rima.KeySize / 8);
            rima.IV = key.GetBytes(rima.BlockSize / 8);
            return rima;
        }

        private static byte[] StreamTransform(byte[] data, ICryptoTransform transform)
        {
            MemoryStream memory = new MemoryStream();
            using (Stream stream = new CryptoStream(memory, transform, CryptoStreamMode.Write))
            {
                stream.Write(data, 0, data.Length);
            }
            return memory.ToArray();
        }

        async public static Task DisplayAskPassword(Page ownerPage, AskPasswordPageType asktype)
        {
            AskPasswordPage passwordpage = new AskPasswordPage(ownerPage, asktype);
            await ownerPage.Navigation.PushModalAsync(passwordpage, false);
        }

        public static bool CheckPassword(string password)
        {
            if (Directory.Exists(StorageHelper.DIRECTORY))
            {
                if (File.Exists(Path.Combine(StorageHelper.DIRECTORY, StorageHelper.CATFILE)))
                    using (var streamReader = new StreamReader(Path.Combine(StorageHelper.DIRECTORY, StorageHelper.CATFILE)))
                    {
                        try
                        {
                            JsonConvert.DeserializeObject<ObservableCollection<Category>>(CryptographyHelper.Decrypt(streamReader.ReadToEnd(), password));
                            return true;
                        }
                        catch (Exception ex)
                        {
                            return false;
                        }
                    }
            }

            return String.IsNullOrEmpty(password);
        }

        public static bool UserNeedPassword()
        {
            return !CheckPassword("");
        }
    }

    public enum AskPasswordPageType
    {
        New,
        Check,
        Login
    }
}