﻿using System;
using System.Linq;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace Arawn.Modules.Cryptography.Pages
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class AskPasswordPage : ContentPage
    {
        public Page OwnerPage { get; set; }
        public AskPasswordPageType AskType { get; set; }

        public AskPasswordPage(Page ownerPage, AskPasswordPageType asktype)
        {
            this.OwnerPage = ownerPage;
            this.AskType = asktype;

            InitializeComponent();

            for (int i = 0; i < 4; i++)
            {
                SPLabels.Children.Add(new Frame()
                {
                    Margin = new Thickness(i != 0 ? 25 : 0, 0, 0, 0),
                    OutlineColor = Color.Transparent,
                    HasShadow = true,
                    Content = new Label()
                    {
                        FontSize = Device.GetNamedSize(NamedSize.Large, typeof(Label)),
                        WidthRequest = 15,
                        HorizontalTextAlignment = TextAlignment.Center,
                        Resources = new ResourceDictionary()
                    }
                });
            }

            for (int i = 0; i < 4; i++)
            {
                StackLayout stack = new StackLayout()
                {
                    Orientation = StackOrientation.Horizontal,
                    HorizontalOptions = LayoutOptions.Center,
                    Margin = new Thickness(0, i != 0 ? 15 : 0, 0, 0)
                };
                for (int j = 0; j < 3; j++)
                {
                    Button btn = new Button()
                    {
                        Text = i != 3 ? $"{i*3 + j + 1}" : (j == 0 ? "C" : (j == 1 ? "0" : "←")),
                        BorderWidth = 0,
                        BorderColor = Color.Transparent,
                        BackgroundColor = Color.Transparent,
                        FontSize = Device.GetNamedSize(NamedSize.Large, typeof(Button)),
                        Margin = new Thickness(j != 0 ? 15 : 0, 0, 0, 0)
                    };
                    if (i != 3 || j == 1) btn.Clicked += Button_Clicked;
                    else if (j == 0) btn.Clicked += ClearClicked;
                    else btn.Clicked += DeleteOneClicked;

                    stack.Children.Add(btn);
                }
                this.SPButtons.Children.Add(stack);
            }
        }

        private void CheckPassword(string password)
        {
            if (this.AskType == AskPasswordPageType.Login)
            {
                MainPage page = ((MainPage)this.OwnerPage);
                if (CryptographyHelper.CheckPassword(password))
                {
                    page.IsAccessGranted = true;
                    CryptographyHelper.PASSWORD = password;
                    this.SendBackButtonPressed();
                }
                else
                {
                    this.ClearClicked(null, null);
                    this.LabelError.Text = "Mot de passe erroné.";
                }
            }
        }

        private void Button_Clicked(object sender, EventArgs e)
        {
            Label label = (Label)this.SPLabels.Children.OfType<Frame>().FirstOrDefault(x => String.IsNullOrEmpty(((Label)x.Content).Text))?.Content;
            if (label != null)
            {
                Label old = (Label)this.SPLabels.Children.OfType<Frame>().FirstOrDefault(x => !String.IsNullOrEmpty(((Label)x.Content).Text) && !((Label)x.Content).Text.Equals("x"))?.Content;
                if (old != null) old.Text = "x";

                label.Text = ((Button)sender).Text;
                label.Resources["Text"] = ((Button)sender).Text;

                if (!this.SPLabels.Children.OfType<Frame>().Any(x => String.IsNullOrEmpty(((Label)x.Content).Text)))
                {
                    string password = string.Join("", this.SPLabels.Children.OfType<Frame>().Select(x => ((Label)x.Content).Resources["Text"]));
                    this.CheckPassword(CryptographyHelper.Encrypt(password, password));
                }
                else
                    this.LabelError.Text = "";
            }
        }

        private void ClearClicked(object sender, EventArgs e)
        {
            foreach (Label label in this.SPLabels.Children.OfType<Frame>().Where(x => !String.IsNullOrEmpty(((Label)x.Content).Text)).Select(x => x.Content))
            {
                label.Text = "";
                label.Resources["Text"] = "";
            }
            this.LabelError.Text = "";
        }

        private void DeleteOneClicked(object sender, EventArgs e)
        {
            Label label = (Label)this.SPLabels.Children.OfType<Frame>().LastOrDefault(x => !String.IsNullOrEmpty(((Label)x.Content).Text))?.Content;
            if (label != null)
            {
                label.Text = "";
                label.Resources["Text"] = "";
                this.LabelError.Text = "";
            }
        }

        private void ContentPage_Disappearing(object sender, EventArgs e)
        {
            if (this.AskType == AskPasswordPageType.Login && !((MainPage)this.OwnerPage).IsAccessGranted)
                System.Environment.Exit(0);
        }
    }
}
