﻿using Arawn.Modules.Cryptography;
using Arawn.Modules.Storage.Models;
using Newtonsoft.Json;
using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;

namespace Arawn.Modules.Storage
{
    public static class StorageHelper
    {
        public static string DIRECTORY = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), "arawn");
        private static string PROPFILE = "1";
        public static string CATFILE = "2";
        private static string IDENFILE = "3";

        public static ObservableCollection<Category> CATEGORIES = new ObservableCollection<Category>();
        public static ObservableCollection<Identifier> IDENTIFIERS = new ObservableCollection<Identifier>();

        public static string THEME;

        public static void SaveTheme()
        {
            if (!Directory.Exists(DIRECTORY))
                Directory.CreateDirectory(DIRECTORY);

            using (var streamWriter = new StreamWriter(Path.Combine(DIRECTORY, PROPFILE), false))
            {
                streamWriter.WriteLine(JsonConvert.SerializeObject(THEME));
            }
        }

        public static void SaveDatas()
        {
            if (!Directory.Exists(DIRECTORY))
                Directory.CreateDirectory(DIRECTORY);
            
            using (var streamWriter = new StreamWriter(Path.Combine(DIRECTORY, CATFILE), false))
            {
                streamWriter.WriteLine(CryptographyHelper.Encrypt(JsonConvert.SerializeObject(CATEGORIES), CryptographyHelper.PASSWORD));
            }
            using (var streamWriter = new StreamWriter(Path.Combine(DIRECTORY, IDENFILE), false))
            {
                streamWriter.WriteLine(CryptographyHelper.Encrypt(JsonConvert.SerializeObject(IDENTIFIERS), CryptographyHelper.PASSWORD));
            }
        }

        public static void LoadTheme()
        {
            THEME = "Light";
            if (Directory.Exists(DIRECTORY))
                if (File.Exists(Path.Combine(DIRECTORY, PROPFILE)))
                    using (var streamReader = new StreamReader(Path.Combine(DIRECTORY, PROPFILE)))
                    {
                        THEME = JsonConvert.DeserializeObject<string>(streamReader.ReadToEnd());
                    }
        }

        public static void LoadDatas()
        {
            CATEGORIES = new ObservableCollection<Category>();
            IDENTIFIERS = new ObservableCollection<Identifier>();

            if (Directory.Exists(DIRECTORY))
            {
                if (File.Exists(Path.Combine(DIRECTORY, CATFILE)))
                    using (var streamReader = new StreamReader(Path.Combine(DIRECTORY, CATFILE)))
                    {
                        CATEGORIES = JsonConvert.DeserializeObject<ObservableCollection<Category>>(CryptographyHelper.Decrypt(streamReader.ReadToEnd(), CryptographyHelper.PASSWORD));
                    }
                if (File.Exists(Path.Combine(DIRECTORY, IDENFILE)))
                    using (var streamReader = new StreamReader(Path.Combine(DIRECTORY, IDENFILE)))
                    {
                        IDENTIFIERS = JsonConvert.DeserializeObject<ObservableCollection<Identifier>>(CryptographyHelper.Decrypt(streamReader.ReadToEnd(), CryptographyHelper.PASSWORD));
                    }
            }

            foreach(Category category in CATEGORIES)
                for (int i = 0; i < category.Identifiers.Count; i++)
                    category.Identifiers[i] = IDENTIFIERS.First(x => x.Id == category.Identifiers[i].Id);
        }
    }
}
