﻿using Arawn.Modules.Storage.Models;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using System.Globalization;
using Arawn.Modules.Cryptography;
using Arawn.Modules.Storage;

namespace Arawn.Pages.Identifiers
{
	[XamlCompilation(XamlCompilationOptions.Compile)]
	public partial class IdentifierEditor : ContentPage
	{
        public IdentifierViewer IdenViewer { get; set; }
        public Identifier Identifier { get; set; }

        public static Identifier CURRENTIDENTIFIER;

        public IdentifierEditor (IdentifierViewer idenviewer, Identifier iden)
        {
            CURRENTIDENTIFIER = iden;

            this.IdenViewer = idenviewer;
            this.Identifier = iden;
            this.BindingContext = this.Identifier;

            InitializeComponent();

            this.ListviewCategories.ItemsSource = StorageHelper.CATEGORIES;
        }

        protected override bool OnBackButtonPressed()
        {
            StorageHelper.SaveDatas();

            return base.OnBackButtonPressed();
        }

        private async void Delete_Tapped(object sender, EventArgs e)
        {
            if (await this.DisplayAlert("Confirmation", "Voulez-vous vraiment supprimer cet identifiant ?", "Oui", "Non"))
            {
                StorageHelper.IDENTIFIERS.Remove(this.Identifier);
                foreach (Category category in StorageHelper.CATEGORIES)
                    category.Identifiers.Remove(this.Identifier);

                StorageHelper.SaveDatas();
                this.IdenViewer.BackTypeEditor = BackPageType.Deleted;

                this.OnBackButtonPressed();
            }
        }

        private void ListviewCategories_ItemSelected(object sender, SelectedItemChangedEventArgs e)
        {
            if (((ListView)sender).SelectedItem != null)
                ((ListView)sender).SelectedItem = null;
        }

        private void SwitchCell_OnChanged(object sender, ToggledEventArgs e)
        {
            Category category = (Category)((SwitchCell)sender).BindingContext;

            if (!((SwitchCell)sender).On && category.Identifiers.Contains(this.Identifier))
                category.Identifiers.Remove(this.Identifier);
            else if(((SwitchCell)sender).On && !category.Identifiers.Contains(this.Identifier))
                category.Identifiers.Add(this.Identifier);
        }
    }

    public class PasswordToClearTextConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return CryptographyHelper.Decrypt(value.ToString(), CryptographyHelper.PASSWORD);
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return CryptographyHelper.Encrypt(value.ToString(), CryptographyHelper.PASSWORD);
        }
    }

    public class IdentifierToSwitchCellValue : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return ((ObservableCollection<Identifier>)value).Contains(IdentifierEditor.CURRENTIDENTIFIER);
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return null;
        }
    }
}